from PIL import Image
from PIL import ImageDraw
import numpy as np
import json
import os, argparse
from tqdm import tqdm

label_colours = [(0,0,0)
                # 0=Background
                ,(128,0,0),(255,0,0),(0,85,0),(170,0,51),(255,85,0)
                # 1=Hat,  2=Hair,    3=Glove, 4=Sunglasses, 5=UpperClothes
                ,(0,0,85),(0,119,221),(85,85,0),(0,85,85),(85,51,0)
                # 6=Dress, 7=Coat, 8=Socks, 9=Pants, 10=Jumpsuits
                ,(52,86,128),(0,128,0),(0,0,255),(51,170,221),(0,255,255)
                # 11=Scarf, 12=Skirt, 13=Face, 14=LeftArm, 15=RightArm
                ,(85,255,170),(170,255,85),(255,255,0),(255,170,0), (0,255,65),(83,53,99),(255,228,225),(139,125,123)
                # 16=LeftLeg, 17=RightLeg, 18=LeftShoe, 19=RightShoe 20=RightHand 21=LeftHand, 22= LeftForearm, 23=RightForearm
                 ,(188,143,143)]
                #24=LowerBody

dict_colours = {(0,0,0):0
                # 0=Background
                ,(128,0,0):1,(255,0,0):2,(0,85,0):3,(170,0,51):4,(255,85,0):5
                # 1=Hat,  2=Hair,    3=Glove, 4=Sunglasses, 5=UpperClothes
                ,(0,0,85):6,(0,119,221):7,(85,85,0):8,(0,85,85):9,(85,51,0):10
                # 6=Dress, 7=Coat, 8=Socks, 9=Pants, 10=Jumpsuits
                ,(52,86,128):11,(0,128,0):12,(0,0,255):13,(51,170,221):14,(0,255,255):15
                # 11=Scarf, 12=Skirt, 13=Face, 14=LeftArm, 15=RightArm
                ,(85,255,170):16,(170,255,85):17,(255,255,0):18,(255,170,0):19,(0,255,65):20,(83,53,99):21
                ,(255,228,225):22,(139,125,123):23,(188,143,143):24}
                # 16=LeftLeg, 17=RightLeg, 18=LeftShoe, 19=RightShoe

def decode_labels(mask, num_images=1, num_classes=22):

    h, w = mask.shape
    outputs = np.zeros((h, w, 3), dtype=np.uint8)
    img = Image.new('RGB', (w, h))
    pixels = img.load()
    for i in range(h):
        for j in range(w):
            pixels[j, i] = label_colours[mask[i,j]]
    outputs = np.array(img)
    return outputs

def is_image_file(filename):

    IMG_EXTENSIONS = [
        '.jpg', '.JPG', '.jpeg', '.JPEG',
        '.png', '.PNG', '.ppm', '.PPM', '.bmp', '.BMP',
    ]
    return any(filename.endswith(extension) for extension in IMG_EXTENSIONS)


def arm_clipper(elbow, shoulder, wrist, size, label):

    shoulder_elbow_vec = (shoulder - elbow)[:2]
    wrist_elbow_vec = (wrist - elbow)[:2]

    k1 = shoulder_elbow_vec[1] / shoulder_elbow_vec[0]
    k2 = wrist_elbow_vec[1] / wrist_elbow_vec[0]

    L_x = np.sqrt(shoulder_elbow_vec.dot(shoulder_elbow_vec))
    L_y = np.sqrt(wrist_elbow_vec.dot(wrist_elbow_vec))

    cos_angle = shoulder_elbow_vec.dot(wrist_elbow_vec) / (L_x * L_y)
    angle = np.arccos(cos_angle)
    angle2 = angle / 2

    if k1 > k2:
        angle_line = angle2 + np.arctan(k2)
    else:
        angle_line = angle2 + np.arctan(k1)

    k = np.tan(angle_line)

    return draw_region(elbow, wrist, size, label, k)


def leg_clipper(hip, knee, size, label):

    mid = np.array([(hip[0]+knee[0])/2, (hip[1]+knee[1])/2])
    hip_knee_vec = np.array([(knee[0]-hip[0]), (knee[1]-hip[1])])

    k = -1 / (hip_knee_vec[1]/hip_knee_vec[0])

    return draw_region(mid, knee, size, label, k)


def draw_region(root, point, size, label, k):

    im = Image.new('L', size=size)
    draw = ImageDraw.Draw(im)

    xmax = (size[1]-root[1])/k + root[0]
    xmin = (-root[1] / k) + root[0]
    ymax = k*(size[0]-root[0]) + root[1]
    ymin = k * (-root[0]) + root[1]

    is_upper = ((k * (point[0] - root[0]) + root[1]) > point[1])

    if xmax >= size[0]:
        if ymin >= 0:
            if is_upper:
                draw.polygon(((0,0), (0,ymin), (size[0],ymax), (size[0], 0)), fill=label, outline=label)
            else:
                draw.polygon(((0,ymin), (0,size[1]), (size[0],size[1]),(size[0], ymax)), fill=label, outline=label)
        else:
            if is_upper:
                draw.polygon(((xmin, 0), (size[0], ymax), (size[0], 0)), fill=label, outline=label)
            else:
                draw.polygon(((0, 0), (0, size[1]), (size[0], size[1]), (size[0], ymax), (xmin, 0)), fill=label,
                             outline=label)

    elif xmax < size[0] and xmax >= 0:
        if xmin >= size[0]:
            if is_upper:
                draw.polygon(((0, 0), (0, size[1]), (xmax, size[1]), (size[0],ymax),(size[0],0)), fill=label, outline=label)
            else:
                draw.polygon(((xmax, size[1]), (size[0], size[1]), (size[0], ymax)), fill=label, outline=label)

        elif xmin<size[0] and xmin >=0:
            if is_upper:
                draw.polygon(((0, 0), (0, size[1]), (xmax, size[1]), (xmin, 0)), fill=label,
                             outline=label)
            else:
                draw.polygon(((xmax, size[1]), (size[0], size[1]), (size[0], 0), (xmin, 0)), fill=label,
                             outline=label)
    elif xmax < 0:
        if ymax >= 0:
            if is_upper:
                draw.polygon(((0, 0), (0, ymin), (size[0], ymax), (size[0], 0)), fill=label,
                         outline=label)
            else:
                draw.polygon(((0, ymin), (0, size[1]), (size[0], size[1]), (size[0], ymax)), fill=label,
                         outline=label)
        else:
            if is_upper:
                draw.polygon(((0, 0), (0, ymin), (xmin, 0)), fill=label,
                         outline=label)
            else:
                draw.polygon(((0, ymin), (0, size[1]), (size[0], size[1]), (size[0], 0),(xmin,0)), fill=label,
                         outline=label)
    return im

if __name__ == '__main__':

    #output_dir = '/Users/admin/Desktop/datagrid/UR_test/parse_convert'
    #img_dir = '/Users/admin/Desktop/datagrid/UR_test/image-parse'
    #json_dir = '/Users/admin/Desktop/datagrid/UR_test/pose-raw'

    parser = argparse.ArgumentParser()
    parser.add_argument('input_parse_dir', type=str)
    parser.add_argument('input_json_dir', type=str)
    parser.add_argument('output_parse_dir', type=str)
    args = parser.parse_args()
    output_dir = args.output_parse_dir
    img_dir = args.input_parse_dir
    json_dir = args.input_json_dir
    if not os.path.exists(output_dir):
        os.mkdir(output_dir)

    img_paths = sorted([f for f in os.listdir(img_dir) if os.path.isfile(os.path.join(img_dir, f))])
    
    for img_filename in tqdm(img_paths):
        if not is_image_file(os.path.join(img_dir, img_filename)):
            continue
        json_name = img_filename[:-4] +'_keypoints.json'
    
        with open(os.path.join(json_dir, json_name), 'r') as f:
            pose_label = json.load(f)
            righthand_data = pose_label['people'][0]['hand_right_keypoints_2d']
            righthand_data = np.array(righthand_data)
            righthand_data = righthand_data.reshape((-1, 3))
            lefthand_data = pose_label['people'][0]['hand_left_keypoints_2d']
            lefthand_data = np.array(lefthand_data)
            lefthand_data = lefthand_data.reshape((-1, 3))
            pose_data = pose_label['people'][0]['pose_keypoints_2d']
            pose_data = np.array(pose_data)
            pose_data = pose_data.reshape((-1, 3))

        right_shoulder = pose_data[2]
        right_elbow = pose_data[3]
        right_wrist = pose_data[4]

        left_shoulder = pose_data[5]
        left_elbow = pose_data[6]
        left_wrist = pose_data[7]

        right_knee = pose_data[9]
        right_hip = pose_data[8]

        left_knee = pose_data[12]
        left_hip = pose_data[11]

        im_parse = Image.open(os.path.join(img_dir, img_filename))
        SIZE = im_parse.size

        parse_array = np.array(im_parse).astype(np.int32)

        right_arm = (parse_array == 15).astype(np.int32)
        left_arm = (parse_array == 14).astype(np.int32)
        right_leg = (parse_array == 17).astype(np.int32)
        left_leg = (parse_array == 16).astype(np.int32)
        pants = (parse_array == 9).astype(np.int32)
        skirt = (parse_array == 12).astype(np.int32)

        parse_array = parse_array - right_arm*15- left_arm*14\
        - pants*9 - skirt*12

        right_arm_clipper = arm_clipper(right_elbow, right_shoulder, right_wrist, SIZE, 1)
        left_arm_clipper = arm_clipper(left_elbow, left_shoulder, left_wrist, SIZE, 1)

        right_arm_clipper = np.array(right_arm_clipper).astype(np.int32)
        left_arm_clipper = np.array(left_arm_clipper).astype(np.int32)

        right_forearm = right_arm * right_arm_clipper

        left_forearm = left_arm * left_arm_clipper

        right_arm = right_arm - right_forearm
        left_arm = left_arm - left_forearm

        right_leg_clipper = leg_clipper(right_hip, right_knee, SIZE, 1)
        left_leg_clipper = leg_clipper(left_hip, left_knee, SIZE, 1)

        right_leg_clipper = np.array(right_leg_clipper).astype(np.int32)
        left_leg_clipper = np.array(left_leg_clipper).astype(np.int32)

        right_clipper_pants = pants * right_leg_clipper
        right_clipper_skirt = skirt * right_leg_clipper
        left_clipper_pants = pants  * left_leg_clipper
        left_clipper_skirt = skirt * left_leg_clipper

        pants_clipper = ((right_clipper_pants + left_clipper_pants)>0).astype(np.int32)
        skirt_clipper = ((right_clipper_skirt + left_clipper_skirt)>0).astype(np.int32)

        pants = pants - pants_clipper
        skirt = skirt - skirt_clipper

        #right_leg = right_leg - (right_shank / 25)
        #left_leg = left_leg - (left_shank / 24)

        #lower_body = pants + skirt - (left_shank / 24 ) - (right_shank / 25)
        parse_array = parse_array + right_arm*15 + left_arm*14\
        + right_forearm*23 + left_forearm*22 + pants*9 + skirt*12\
        + pants_clipper*24 + skirt_clipper*24

        im_parse = Image.fromarray(parse_array.astype(np.int32))
        im_parse.save(os.path.join(output_dir, img_filename))

        img_filename = 'vis_' + img_filename
        parse_array = decode_labels(parse_array.astype(np.int32))
        im_parse = Image.fromarray(parse_array)
        im_parse.save(os.path.join(output_dir, img_filename))






import os, argparse, shutil
import glob
from tqdm import tqdm
from PIL import Image

parser = argparse.ArgumentParser()
parser.add_argument('input_dir', type=str)
parser.add_argument('output_dir', type=str)
parser.add_argument('split_size', type=int)
args = parser.parse_args()
input_dir = args.input_dir
output_dir = args.output_dir

image_paths = glob.glob(input_dir+'/*.jpg')
image_paths.extend(glob.glob(input_dir+'/*.png'))

over_dir = os.path.join(output_dir, 'over_%d' % args.split_size)
under_dir = os.path.join(output_dir, 'under_%d' % args.split_size)
os.makedirs(over_dir, exist_ok=True)
os.makedirs(under_dir, exist_ok=True)

for image_path in tqdm(image_paths):
    try:
        image = Image.open(image_path)
        w, h = image.size
        if w>args.split_size:
            new_image_path = os.path.join(over_dir, os.path.basename(image_path))
        else:
            new_image_path = os.path.join(under_dir, os.path.basename(image_path))
        shutil.copyfile(image_path,new_image_path)
    except:
        print(image_path,': failed')
        pass



import os, argparse
import numpy as np
from PIL import Image, ImageFile
from tqdm import tqdm
import cv2
from dlib import get_frontal_face_detector, shape_predictor
from imutils import face_utils


def calc_black_region_ratio(PIL_img):
    img_array = np.asarray(PIL_img)
    ratio = np.mean((img_array[:, :, 0] == 0) & (img_array[:, :, 1] == 0) & (img_array[:, :, 2] == 0))
    return ratio

def points2landmarks(points):
    # see the following page for definition of annotation points
    # https://ibug.doc.ic.ac.uk/resources/300-W/
    jaw = points[0:17]
    right_eyebrow = points[17:22]
    left_eyebrow = points[22:27]
    nose = points[27:36]
    right_eye = points[36:42]
    left_eye = points[42:48]
    mouth = points[48:60]
    lip = points[60:68]

    landmarks = {
        'jaw': jaw,
        'right_eyebrow': right_eyebrow,
        'left_eyebrow': left_eyebrow,
        'nose': nose,
        'right_eye': right_eye,
        'left_eye': left_eye,
        'mouth': mouth,
        'lip': lip
    }
    return landmarks

class FaceLandmarkDetector(object):
    def __init__(self, landmark_predictor_path=None):
        if landmark_predictor_path:
            self.landmark_predictor_path = landmark_predictor_path
        else:
            self.landmark_predictor_path = "shape_predictor_68_face_landmarks.dat"

        self.detector = get_frontal_face_detector()
        self.predictor = shape_predictor(self.landmark_predictor_path)

    def detect(self, img, return_type='landmarks'):
        """
        detect face landmarks of a given image

        Parameters
        ----------
        img: PIL.Image, RGB

        Returns
        -------
        landmarks : dict of np.array (None, 2)
            e.g. landmarks['jaw'][:, 0] are the x-coordinates of jaw
            landmarks, and landmars['jaw'][:, 1] are the y-coorinates.
        """
        landmarks = []
        points_list = []
        dets, _, _ = self.detector.run(img, 0)
        if len(dets)==0:
            return None
        for rect in dets:
            points = self.predictor(img, rect)
            points = face_utils.shape_to_np(points)
            points_list.append(points)
            landmarks.append(points2landmarks(points))

        if return_type == 'points':
            return points_list
        return landmarks

class FaceCropper:
    def __init__(self, resolution, mirror_padding, check_resolution=False, upper_limit_of_black_region_ratio=0.05):
        self.resolution = resolution
        self.check_resolution = check_resolution
        self.mirror_padding = mirror_padding
        self.upper_limit_of_black_region_ratio = upper_limit_of_black_region_ratio
        self.face_landmark_detector = FaceLandmarkDetector()
        self.n_all = 0
        self.n_small = 0
        self.n_detected = 0
        self.n_not_detected = 0
        self.m_all_face = 0
        self.m_low_resolution = 0
        self.m_high_black_region_ratio = 0
        self.m_ok_face = 0

    def _crop_face_by_landmarks(self, input_img, landmark_points):
        input_img_array = np.asarray(input_img)
        left_eye = landmark_points[36:42].mean(axis=0)
        right_eye = landmark_points[42:48].mean(axis=0)
        mouth_left = landmark_points[48]
        mouth_right = landmark_points[54]

        center_eyes = (left_eye + right_eye) / 2 + 0.5
        mouth_center = (mouth_left + mouth_right) / 2 + 0.5
        left_eye_to_right_eye = right_eye - left_eye
        eye_to_mouth = mouth_center - center_eyes

        new_x = left_eye_to_right_eye - np.array([-eye_to_mouth[1], eye_to_mouth[0]])
        new_x /= np.linalg.norm(new_x)
        new_x *= max(np.linalg.norm(left_eye_to_right_eye) * 2.0, np.linalg.norm(eye_to_mouth) * 1.8)
        length = np.linalg.norm(new_x)

        if self.check_resolution:
            if length * 2 <= self.resolution:
                return None

        # area expantion and translation to center
        move_xy = np.array((input_img.size[0] / 2, input_img.size[1] / 2))
        translated_img_size = (input_img.size[0] * 2, input_img.size[1] * 2)
        if self.mirror_padding == True:
          pad_w = input_img_array.shape[1]
          pad_h = input_img_array.shape[0]
          translated_img_array = np.pad(input_img_array, ((int(pad_h/2), int(pad_h/2)), (int(pad_w/2), int(pad_w/2)), (0, 0)), mode='reflect')
        else:
          translated_img_array = np.zeros((input_img.size[1] * 2, input_img.size[0] * 2, 3), dtype=np.uint8)
        index0 = (int(move_xy[1]), int(move_xy[1] + input_img.size[1]))
        index1 = (int(move_xy[0]), int(move_xy[0] + input_img.size[0]))
        translated_img_array[index0[0]:index0[1], index1[0]:index1[1] :] = input_img_array

        # rotate input_image on face_center
        face_center = center_eyes + eye_to_mouth * 0.1 + move_xy # translate face_center
        angle = 90 - np.degrees(np.arctan2(*new_x.tolist()))
        rotation_matrix = cv2.getRotationMatrix2D(tuple(face_center.tolist()), angle, scale=1)
        rotated_translated_img_array = cv2.warpAffine(translated_img_array, rotation_matrix, translated_img_size,
                                     flags=cv2.INTER_LANCZOS4)
        rotated_translated_img = Image.fromarray(rotated_translated_img_array)

        # crop and resize facial image
        cropped_points = [face_center[0] - length,
                          face_center[1] - length,
                          face_center[0] + length,
                          face_center[1] + length
                          ]
        cropped_img = rotated_translated_img.crop(cropped_points)
        return cropped_img

    def crop_face_from_path(self, image_path):
        '''
        crop image from given image_path.
        Input
            image_path: input image path  (dtype=str)
        Output
            cropped_image: cropped image (dtype=PIL.image)
        '''
        input_img = Image.open(image_path).convert("RGB")
        return self.crop_face_from_image(input_img)
    
    def crop_face_from_image(self, input_img):
        '''
        crop image from given input_img.
        Input
            input_img: input image (dtype=PIL.image)
        Output
            cropped_image: cropped image (dtype=PIL.image)
        '''
        self.n_all += 1

        input_img_array = np.asarray(input_img)
        flattened_landmarks_list = self.face_landmark_detector.detect(input_img_array, return_type='points')
        if flattened_landmarks_list is None:
#            print(image_path)
#            print("landmarks are not detected")
            self.n_not_detected += 1
            return None
        self.n_detected += 1

        cropped_imgs = []
        self.m_all_face += len(flattened_landmarks_list)
        for flattened_landmarks in flattened_landmarks_list:
            cropped_img = self._crop_face_by_landmarks(input_img, flattened_landmarks)
            if self.check_resolution:
                if cropped_img is None:
                    self.m_low_resolution += 1
 #                   print(image_path)
 #                   print('low resolution')

            if cropped_img is not None:
                black_region_ratio = calc_black_region_ratio(cropped_img)
                if black_region_ratio > self.upper_limit_of_black_region_ratio:
                    self.m_high_black_region_ratio += 1
                    cropped_img = None
                elif self.resolution == -1:
                    pass
                else:
                    self.m_ok_face += 1
                    cropped_img = cropped_img.resize((self.resolution, self.resolution),
                                                     resample=Image.LANCZOS)

            cropped_imgs.append(cropped_img)
        return cropped_imgs

    def summary(self):
        print('Resolution: %d' % self.resolution)
        print('Upper limit of black region ratio %.0f%%' % (self.upper_limit_of_black_region_ratio * 100))
        print()
        print('All images: %d' % self.n_all)
        print('Small images: %d(%.2f%%)' % (self.n_small, self.n_small / self.n_all * 100))
        print('Detected images: %d(%.2f%%)' % (self.n_detected, self.n_detected / self.n_all * 100))
        print('Not detected images: %d(%.2f%%)' % (self.n_not_detected, self.n_not_detected / self.n_all * 100))
        print()
        print('All faces: %d(%.2f%% for Detected images) ' % (self.m_all_face, self.m_all_face / self.n_detected * 100))
        print('OK faces: %d(%.2f%%)' % (self.m_ok_face, self.m_ok_face / self.m_all_face * 100))
        print('Low resolution faces: %d(%.2f%%)' % (self.m_low_resolution, self.m_low_resolution / self.m_all_face * 100))
        print('High black region ratio faces: %d(%.2f%%)' % (self.m_high_black_region_ratio, self.m_high_black_region_ratio / self.m_all_face * 100))
        print()
        print('OK faces / All images: %.2f%% ' % (self.m_ok_face / self.n_all * 100))

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('input_dir', type=str)
    parser.add_argument('output_dir', type=str)
    parser.add_argument('resolution', type=int, help='crop resolution. If you set resolution = -1, you can skip resizing process.')
    parser.add_argument('--mirror_padding', action='store_true', default=False, help='If you add this option, mirror padding is excecuted.')
    args = parser.parse_args()

    image_names = [f for f in os.listdir(args.input_dir) if f.endswith(('.png', '.jpg', '.jpeg', '.JPG'))]
    if not os.path.exists(args.output_dir):
        os.mkdir(args.output_dir)
    face_cropper = FaceCropper(args.resolution, check_resolution=False, mirror_padding=args.mirror_padding, upper_limit_of_black_region_ratio=1.)

    for image_name in image_names:
        image_path = os.path.join(args.input_dir, image_name)
        cropped_imgs = face_cropper.crop_face_from_path(image_path)
        if cropped_imgs is None:
            continue

        for i_face, cropped_img in enumerate(cropped_imgs):
            if cropped_img is not None:
                output_img_path = os.path.join(args.output_dir, image_name[:-4] + '-%d.png' % i_face)
                cropped_img.save(output_img_path)
    face_cropper.summary()

if __name__ == '__main__':
    main()
import os, argparse
import numpy as np
from PIL import Image, ImageFile
from tqdm import tqdm
import cv2
from dlib import get_frontal_face_detector, shape_predictor
from imutils import face_utils


def calc_black_region_ratio(PIL_img):
    img_array = np.asarray(PIL_img)
    ratio = np.mean((img_array[:, :, 0] == 0) & (img_array[:, :, 1] == 0) & (img_array[:, :, 2] == 0))
    return ratio

def points2landmarks(points):
    # see the following page for definition of annotation points
    # https://ibug.doc.ic.ac.uk/resources/300-W/
    jaw = points[0:17]
    right_eyebrow = points[17:22]
    left_eyebrow = points[22:27]
    nose = points[27:36]
    right_eye = points[36:42]
    left_eye = points[42:48]
    mouth = points[48:60]
    lip = points[60:68]

    landmarks = {
        'jaw': jaw,
        'right_eyebrow': right_eyebrow,
        'left_eyebrow': left_eyebrow,
        'nose': nose,
        'right_eye': right_eye,
        'left_eye': left_eye,
        'mouth': mouth,
        'lip': lip
    }
    return landmarks

class FaceLandmarkDetector(object):
    def __init__(self, landmark_predictor_path=None):
        if landmark_predictor_path:
            self.landmark_predictor_path = landmark_predictor_path
        else:
            self.landmark_predictor_path = "shape_predictor_68_face_landmarks.dat"

        self.detector = get_frontal_face_detector()
        self.predictor = shape_predictor(self.landmark_predictor_path)

    def detect(self, img, return_type='landmarks'):
        """
        detect face landmarks of a given image

        Parameters
        ----------
        img: PIL.Image, RGB

        Returns
        -------
        landmarks : dict of np.array (None, 2)
            e.g. landmarks['jaw'][:, 0] are the x-coordinates of jaw
            landmarks, and landmars['jaw'][:, 1] are the y-coorinates.
        """
        landmarks = []
        points_list = []
        dets, _, _ = self.detector.run(img, 0)
        if len(dets)==0:
            return None
        for rect in dets:
            points = self.predictor(img, rect)
            points = face_utils.shape_to_np(points)
            points_list.append(points)
            landmarks.append(points2landmarks(points))

        if return_type == 'points':
            return points_list
        return landmarks

class FaceCropper:
    def __init__(self, resolution, mirror_padding, check_resolution=False, upper_limit_of_black_region_ratio=0.05):
        self.resolution = resolution
        self.check_resolution = check_resolution
        self.mirror_padding = mirror_padding
        self.upper_limit_of_black_region_ratio = upper_limit_of_black_region_ratio
        self.face_landmark_detector = FaceLandmarkDetector()
        self.n_all = 0
        self.n_small = 0
        self.n_detected = 0
        self.n_not_detected = 0
        self.m_all_face = 0
        self.m_low_resolution = 0
        self.m_high_black_region_ratio = 0
        self.m_ok_face = 0

    def _crop_face_by_landmarks(self, input_img, landmark_points):
        input_img_array = np.asarray(input_img)
        left_eye = landmark_points[36:42].mean(axis=0)
        right_eye = landmark_points[42:48].mean(axis=0)
        mouth_left = landmark_points[48]
        mouth_right = landmark_points[54]

        center_eyes = (left_eye + right_eye) / 2 + 0.5
        mouth_center = (mouth_left + mouth_right) / 2 + 0.5
        left_eye_to_right_eye = right_eye - left_eye
        eye_to_mouth = mouth_center - center_eyes

        new_x = left_eye_to_right_eye - np.array([-eye_to_mouth[1], eye_to_mouth[0]])
        new_x /= np.linalg.norm(new_x)
        new_x *= max(np.linalg.norm(left_eye_to_right_eye) * 2.0, np.linalg.norm(eye_to_mouth) * 1.8)
        length = np.linalg.norm(new_x)

        if self.check_resolution:
            if length * 2 <= self.resolution:
                return None

        # area expantion and translation to center
        move_xy = np.array((input_img.size[0] / 2, input_img.size[1] / 2))
        translated_img_size = (input_img.size[0] * 2, input_img.size[1] * 2)
        if self.mirror_padding == True:
          pad_w = input_img_array.shape[1]
          pad_h = input_img_array.shape[0]
          translated_img_array = np.pad(input_img_array, ((int(pad_h/2), int(pad_h/2)), (int(pad_w/2), int(pad_w/2)), (0, 0)), mode='reflect')
        else:
          translated_img_array = np.zeros((input_img.size[1] * 2, input_img.size[0] * 2, 3), dtype=np.uint8)
        index0 = (int(move_xy[1]), int(move_xy[1] + input_img.size[1]))
        index1 = (int(move_xy[0]), int(move_xy[0] + input_img.size[0]))
        translated_img_array[index0[0]:index0[1], index1[0]:index1[1] :] = input_img_array

        # rotate input_image on face_center
        face_center = center_eyes + eye_to_mouth * 0.1 + move_xy # translate face_center
        angle = 90 - np.degrees(np.arctan2(*new_x.tolist()))
        rotation_matrix = cv2.getRotationMatrix2D(tuple(face_center.tolist()), angle, scale=1)
        rotated_translated_img_array = cv2.warpAffine(translated_img_array, rotation_matrix, translated_img_size,
                                     flags=cv2.INTER_LANCZOS4)
        rotated_translated_img = Image.fromarray(rotated_translated_img_array)

        # crop and resize facial image
        cropped_points = [face_center[0] - length,
                          face_center[1] - length,
                          face_center[0] + length,
                          face_center[1] + length
                          ]
        cropped_img = rotated_translated_img.crop(cropped_points)
        return cropped_img

    def crop_face_from_path(self, image_path):
        '''
        crop image from given image_path.
        Input
            image_path: input image path  (dtype=str)
        Output
            cropped_image: cropped image (dtype=PIL.image)
        '''
        input_img = Image.open(image_path).convert("RGB")
        return self.crop_face_from_image(input_img)
    
    def crop_face_from_image(self, input_img):
        '''
        crop image from given input_img.
        Input
            input_img: input image (dtype=PIL.image)
        Output
            cropped_image: cropped image (dtype=PIL.image)
        '''
        self.n_all += 1

        input_img_array = np.asarray(input_img)
        flattened_landmarks_list = self.face_landmark_detector.detect(input_img_array, return_type='points')
        if flattened_landmarks_list is None:
#            print(image_path)
#            print("landmarks are not detected")
            self.n_not_detected += 1
            return None
        self.n_detected += 1

        cropped_imgs = []
        self.m_all_face += len(flattened_landmarks_list)
        for flattened_landmarks in flattened_landmarks_list:
            cropped_img = self._crop_face_by_landmarks(input_img, flattened_landmarks)
            if self.check_resolution:
                if cropped_img is None:
                    self.m_low_resolution += 1
 #                   print(image_path)
 #                   print('low resolution')

            if cropped_img is not None:
                black_region_ratio = calc_black_region_ratio(cropped_img)
                if black_region_ratio > self.upper_limit_of_black_region_ratio:
                    self.m_high_black_region_ratio += 1
                    cropped_img = None
                elif self.resolution == -1:
                    pass
                else:
                    self.m_ok_face += 1
                    cropped_img = cropped_img.resize((self.resolution, self.resolution),
                                                     resample=Image.LANCZOS)

            cropped_imgs.append(cropped_img)
        return cropped_imgs

    # def summary(self):
    #     print('Resolution: %d' % self.resolution)
    #     print('Upper limit of black region ratio %.0f%%' % (self.upper_limit_of_black_region_ratio * 100))
    #     print()
    #     print('All images: %d' % self.n_all)
    #     print('Small images: %d(%.2f%%)' % (self.n_small, self.n_small / self.n_all * 100))
    #     print('Detected images: %d(%.2f%%)' % (self.n_detected, self.n_detected / self.n_all * 100))
    #     print('Not detected images: %d(%.2f%%)' % (self.n_not_detected, self.n_not_detected / self.n_all * 100))
    #     print()
    #     print('All faces: %d(%.2f%% for Detected images) ' % (self.m_all_face, self.m_all_face / self.n_detected * 100))
    #     print('OK faces: %d(%.2f%%)' % (self.m_ok_face, self.m_ok_face / self.m_all_face * 100))
    #     print('Low resolution faces: %d(%.2f%%)' % (self.m_low_resolution, self.m_low_resolution / self.m_all_face * 100))
    #     print('High black region ratio faces: %d(%.2f%%)' % (self.m_high_black_region_ratio, self.m_high_black_region_ratio / self.m_all_face * 100))
    #     print()
    #     print('OK faces / All images: %.2f%% ' % (self.m_ok_face / self.n_all * 100))

def crop_image(img, resolution, if_mirror_padding=False):
    # img is pil or path
    face_cropper = FaceCropper(resolution, check_resolution=False, mirror_padding=if_mirror_padding, upper_limit_of_black_region_ratio=1.)
    cropped_img = face_cropper.crop_face(img)
    return cropped_img

def crop_image_from_path(img_path, resolution, if_mirror_padding=False):
    face_cropper = FaceCropper(resolution, check_resolution=False, mirror_padding=if_mirror_padding, upper_limit_of_black_region_ratio=1.)
    cropped_img = face_cropper.crop_face_from_path(img)
    return cropped_img


import os                                                                                                                                                                                                   
from PIL import Image
from tqdm import tqdm
import argparse

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='resize oversize images')
    parser.add_argument('image_dir', type=str, help='path of input image dir')
    parser.add_argument('image_out_dir', type=str, help='path of ouput image dir')
    parser.add_argument('resolution', type=int, help='path of ouput image dir')
    args = parser.parse_args()

    image_out_dir = args.image_out_dir
    if not os.path.isdir(image_out_dir):
        os.mkdir(image_out_dir)

    image_dir = args.image_dir
    if not os.path.isabs(image_dir):
        image_dir = os.path.join(os.getcwd(), image_dir)

    image_names = [f for f in os.listdir(image_dir) if f.endswith(('.jpg', '.png'))]

    for image_name in tqdm(image_names):
        image_path = os.path.join(image_dir, image_name)
        image_out_path = os.path.join(image_out_dir, image_name)
        img = Image.open(image_path)
        resized = img.resize((args.resolution, args.resolution), Image.LANCZOS)
        resized.save(image_out_path[:-4]+'.png')



import os
import argparse
from tqdm import tqdm
import cv2
from PIL import Image
import numpy as np

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('in_image_dir', type=str )
    parser.add_argument('in_image_parse_dir', type=str)
    parser.add_argument('out_image_and_parse_dir', type=str)
    parser.add_argument('--alpha', type=float, default=0.60)
    parser.add_argument('--debug', action='store_true')
    args = parser.parse_args()

    in_image_dir = args.in_image_dir
    in_image_parse_dir = args.in_image_parse_dir
    out_image_and_parse_dir = args.out_image_and_parse_dir

    if not os.path.exists(out_image_and_parse_dir):
        os.mkdir(out_image_and_parse_dir)

    image_names = sorted([f for f in os.listdir(in_image_dir) if f.endswith(('.jpg', '.png'))])
    #image_parse_names = sorted([f for f in os.listdir(in_image_parse_dir) if f.endswith(('.jpg', '.png'))])

    n_preview = 3
    for image_name in tqdm(image_names):
        img = cv2.imread( os.path.join(in_image_dir, image_name) )

        if( in_image_parse_dir.split("/")[-1] == "image-parse3" ):
            img_parse = cv2.imread( os.path.join(in_image_parse_dir, "vis_" + image_name) )
        else:
            img_parse = cv2.imread( os.path.join(in_image_parse_dir, image_name) )

        if( args.debug ):
            if(n_preview > 0):
                n_preview -= 1
                print( "img.shape :", img.shape )
                print( "img_parse.shape :", img_parse.shape )

        blended_img = cv2.addWeighted( img, args.alpha, img_parse, 1.0 - args.alpha, 0)
        #cv2.imwrite( os.path.join(out_image_and_parse_dir, image_name), blended_img )

        base_img = Image.new( "RGB", (3*img.shape[1], img.shape[0]), (0, 0, 0) ) 
        base_img.paste( Image.fromarray(cv2.cvtColor(blended_img, cv2.COLOR_BGR2RGB)), (0, 0) )
        base_img.paste( Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB)), (1*img.shape[1], 0) )
        base_img.paste( Image.fromarray(cv2.cvtColor(img_parse, cv2.COLOR_BGR2RGB)), (2*img.shape[1], 0) )
        base_img.save( os.path.join(out_image_and_parse_dir, image_name) )



from multiprocessing import Pool
import os, argparse
from tqdm import tqdm
import cv2 as cv

def margin_crop(img_name):
    img = cv.imread(os.path.join(image_dir_shared, img_name))
    L2 = int(L_shared/2)
    img = img[L_shared:,L2:-L2]
    cv.imwrite(os.path.join(output_dir_shared,img_name), img)

def initializer(image_dir,L,output_dir):
    global image_dir_shared,L_shared,output_dir_shared
    image_dir_shared = image_dir
    L_shared = L
    output_dir_shared = output_dir

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--image_dir', type=str)
    parser.add_argument('--L', type=int)
    parser.add_argument('--output_dir', type=str)
    parser.add_argument('--workers', type=int, default=10)
    args = parser.parse_args()
    image_dir = args.image_dir
    L = args.L
    output_dir = args.output_dir
    image_names = [f for f in os.listdir(image_dir) if f.endswith(('.png', '.jpg'))]
    image_names = sorted(image_names)

    if not os.path.exists(output_dir):
        os.mkdir(output_dir)

    pool = Pool(args.workers,initializer=initializer,initargs=(image_dir,L,output_dir))
    with tqdm(total=len(image_names)) as t:
        for _ in pool.imap_unordered(margin_crop, image_names):
            t.update(1)
        pool.close()
        pool.join()
        


from setuptools import setup, find_packages

VERSION = '0.0.3' 
DESCRIPTION = 'Common tools'
LONG_DESCRIPTION = 'Some common tools for image and video processing'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="dg_util", 
        version=VERSION,
        author="DataGrid",
        author_email="",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['numpy', 'tqdm', 'opencv-python'], # add any additional packages that 
        # needs to be installed along with your package. Eg: 'caer'
        
        keywords=['python', 'Tools for Internal Use Only'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            # "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: POSIX :: Linux",
            "Operating System :: Unix"
        ]
)